#11.04.10 00.00 chap3.r
#copyright 2010 by Robert A LaBudde, all rights reserved
#Chapter 3 examples for Intro to Bootstrap
#created: 11.04.10 by r.a. labudde
#changes: 

set.seed(1) #for reproducible results

#deformation force required to puncture test specimens for 40 lots of surimi
x<- c(41.28, 45.16, 34.75, 40.76, 43.61, 39.05, 41.20, 41.02, 41.33, 40.61, 40.49, 41.77, 42.07, 
  44.83, 29.12, 45.59, 41.95, 45.78, 42.89, 40.42, 49.31, 44.01, 34.87, 38.60, 39.63, 38.52, 38.52,
  43.95, 49.08, 50.52, 43.85, 40.64, 45.86, 41.25, 50.35, 45.18, 39.67, 43.89, 43.89, 42.16)
summary(x)
c(var(x), sd(x))  #variance and standard deviation
shapiro.test(x) #check normality  

#assuming normality
n<- length(x) #sample size
n
mean(x) + qt(0.975, n-1)*sd(x)*c(-1,+1)/sqrt(n)   #student t based 95% CI

#Efron percentile method
set.seed(1)   #for reproducibility
u0<- mean(x)  #mean of original sample
Sh<- sd(x)/sqrt(n)
thetas<- NULL
for (i in 1:1000) { #bootstrap resampling
  xx<- sample(x, n, replace=TRUE)   #new resample
  u<- mean(xx)  #estimate
  thetas[i]<- u #save
}
c(u0, mean(thetas)) #how does mean of bootstrap estimates match sample mean?
quantile(thetas, probs=c(0.025,0.975))  #Efron percentile CI


#bootstrap percentile t
set.seed(1)   #for reproducibility
u0<- mean(x)  #mean of original sample
Sh<- sd(x)/sqrt(n)
thetas<- NULL
tstar<- NULL
for (i in 1:1000) { #bootstrap resampling
  xx<- sample(x, n, replace=TRUE)   #new resample
  u<- mean(xx)  #estimate
  thetas[i]<- u #save
  tstar[i]<- (u - u0)/(sd(xx)/sqrt(n))  #pivotal quantity
}
c(u0, mean(thetas))   #compare sample mean to mean of bootstrap sampling distribution
c(Sh, sd(thetas))     #compare standard error from sample to standard error estimate from bootstrap distribution
summary(tstar)    #look symmetric & normal?
quantile(tstar, probs=c(0.025,0.975))   #quantiles from bootstrap percentile t
qt(c(0.025,0.975), n-1)   #quantiles from student t distribution
u0 + quantile(tstar, probs=c(0.025,0.975))*Sh  #bootstrap percentile t confidence interval
u0 + qt(c(0.025,0.975), n-1)*Sh    #student t confidence interval for comparison

set.seed(1)   #reproducibility
require('bootstrap')
boott(x, theta=mean, sdfun=function(x,nbootsd,theta,...) {sqrt(var(x)/length(x))}, 
  nboott=1000, perc=c(0.025,0.975)) #bootstrap percentile t 95% C.I.

#double bootstrap
set.seed(1)   #reproducibility
boott(x, theta=mean, nbootsd=100, nboott=1000, perc=c(0.025,0.975)) #bootstrap percentile t 95% C.I. with nested bootstrap

#bias corrected bootstrap
set.seed(1)
fboot<- function(x, i) mean(x[i])   #compute estimate given data x and index set i

bootTheta <- function (x, nBoot, theta, ...) {
#performs bootstrap resampling to get array of thetastar values
  n <- length(x)
  bootsam <- matrix(sample(x, size = n * nBoot, replace = TRUE), 
     nrow = nBoot) #bootstrap samples, nboot sets of size n
  thetastar <- apply(bootsam, 1, theta, ...) #calc thetastar estimates for each resample
  return (thetastar)
}

ciBCa <- function (accel, thetahat, thetastar, alpha) {
#IN:  accel = acceleration constant from original sample
#     thetahat = estimate for original sample
#     thetastar[] = bootstrap estimates
#     alpha = confidence interval probs
  nBoot <- length(thetastar) #number of bootstrap resamples
  z0 <- qnorm(sum(thetastar < thetahat)/nBoot) #quantile for bias
  zCI <- qnorm(alpha) #quantiles for c.i.
  p <- pnorm(z0 + (z0 + zCI)/(1 - accel*(z0 + zCI)) ) #prob for BCa adjustment
  ind <- trunc(p*nBoot) #index for quantile
  CIp <- quantile(thetastar, probs=p)
  CI <- sort(thetastar)[ind]
  return (list(accel=accel, z0 = z0, zCI = zCI, probs =p, CI = CI, CIp = CIp))
}

thetastar<- bootTheta(x, nBoot=1000, fboot)  #get resample estimates
ciBCa(accel=0, mean(x), thetastar, c(0.025,0.975))  #BC confidence interval via accel=0


#BCa and ABC
set.seed(1)   #for reproducibility
require('boot')
fboot<- function(x, i) mean(x[i])   #compute estimate given data x and index set i
bs<- boot(x, fboot, R=1000)   #generate bootstrap estimates
boot.ci(bs, type='bca', conf=0.95)  #BCa method 95% C.I.
fabc<- function(x, w) w%*%x   #ABC uses weighted average
abc.ci(x, fabc, conf=0.95)  #ABC method C.I.


#EXERCISE 1: Mice weights
require('boot')
x<- c(82, 107, 93)  #weights of mice
c(mean(x), sd(x), sd(x)/sqrt(3))  #mean, SD, SE(mean)
c(mean(x)-qt(0.975,2)*sd(x)/sqrt(3), mean(x)+qt(0.975,2)*sd(x)/sqrt(3))  #student t interval

set.seed(1)
fboot<- function(x, i) mean(x[i])   #compute estimate given data x and index set i
bs<- boot(x, fboot, R=1000)   #generate bootstrap estimates
boot.ci(bs, conf=0.95)  #various methods 95% C.I.
fabc<- function(x, w) w%*%x   #ABC uses weighted average
abc.ci(x, fabc, conf=0.95)  #ABC method C.I.

n<- length(x)
u0<- mean(x)  #mean of original sample
Sh<- sd(x)/sqrt(n)  #SE(mean)
thetas<- NULL
tstar<- NULL
for (i in 1:1000) { #bootstrap resampling
  xx<- sample(x, n, replace=TRUE)   #new resample
  u<- mean(xx)  #estimate
  thetas[i]<- u #save
  tstar[i]<- (u - u0)/(sd(xx)/sqrt(n))  #pivotal quantity
}
c(u0, mean(thetas))   #compare sample mean to mean of bootstrap sampling distribution
c(Sh, sd(thetas))     #compare standard error from sample to standard error estimate from bootstrap distribution
summary(tstar)    #look symmetric & normal?
quantile(tstar, probs=c(0.025,0.975))   #quantiles from bootstrap percentile t
qt(c(0.025,0.975), n-1)   #quantiles from student t distribution
u0 + quantile(tstar, probs=c(0.025,0.975))*Sh  #bootstrap percentile t confidence interval
u0 + qt(c(0.025,0.975), n-1)*Sh    #student t confidence interval for comparison

#EXERCISE 3: Aflatoxin residues
require('boot')
x<- c(4.94,5.06,4.53,5.07,4.99,5.16,4.38,4.43,4.93,4.72,4.92,4.96)
n<- length(x) #number data
u0<- mean(x)  #mean of original sample
Sh<- sd(x)/sqrt(n)  #SE(mean)
c(u0, sd(x), Sh)  #mean, SD, SE(mean)
c(u0-qt(0.975,n-1)*Sh, u0+qt(0.975,n-1)*Sh)  #student t interval

set.seed(1)
fboot<- function(x, i) mean(x[i])   #compute estimate given data x and index set i
bs<- boot(x, fboot, R=1000)   #generate bootstrap estimates
boot.ci(bs, conf=0.95)  #various methods 95% C.I.

fabc<- function(x, w) w%*%x   #ABC uses weighted average
abc.ci(x, fabc, conf=0.95)  #ABC method C.I.

thetas<- NULL
tstar<- NULL
for (i in 1:1000) { #bootstrap resampling
  xx<- sample(x, n, replace=TRUE)   #new resample
  u<- mean(xx)  #estimate
  thetas[i]<- u #save
  tstar[i]<- (u - u0)/(sd(xx)/sqrt(n))  #pivotal quantity
}
c(u0, mean(thetas))   #compare sample mean to mean of bootstrap sampling distribution
c(Sh, sd(thetas))     #compare standard error from sample to standard error estimate from bootstrap distribution
summary(tstar)    #look symmetric & normal?
quantile(tstar, probs=c(0.025,0.975))   #quantiles from bootstrap percentile t
qt(c(0.025,0.975), n-1)   #quantiles from student t distribution
u0 + quantile(tstar, probs=c(0.025,0.975))*Sh  #bootstrap percentile t confidence interval
u0 + qt(c(0.025,0.975), n-1)*Sh    #student t confidence interval for comparison

#EXERCISE 4: Razor blade sharpness
require('boot')
x<- c(8.5,13.9,7.4,10.3,15.7,4.0)
n<- length(x) #number data
u0<- mean(x)  #mean of original sample
Sh<- sd(x)/sqrt(n)  #SE(mean)
c(u0, sd(x), Sh)  #mean, SD, SE(mean)
c(u0-qt(0.975,n-1)*Sh, u0+qt(0.975,n-1)*Sh)  #student t interval

set.seed(1)
fboot<- function(x, i) mean(x[i])   #compute estimate given data x and index set i
bs<- boot(x, fboot, R=1000)   #generate bootstrap estimates
boot.ci(bs, conf=0.95)  #various methods 95% C.I.

fabc<- function(x, w) w%*%x   #ABC uses weighted average
abc.ci(x, fabc, conf=0.95)  #ABC method C.I.

thetas<- NULL
tstar<- NULL
for (i in 1:1000) { #bootstrap resampling
  xx<- sample(x, n, replace=TRUE)   #new resample
  u<- mean(xx)  #estimate
  thetas[i]<- u #save
  tstar[i]<- (u - u0)/(sd(xx)/sqrt(n))  #pivotal quantity
}
c(u0, mean(thetas))   #compare sample mean to mean of bootstrap sampling distribution
c(Sh, sd(thetas))     #compare standard error from sample to standard error estimate from bootstrap distribution
summary(tstar)    #look symmetric & normal?
quantile(tstar, probs=c(0.025,0.975))   #quantiles from bootstrap percentile t
qt(c(0.025,0.975), n-1)   #quantiles from student t distribution
u0 + quantile(tstar, probs=c(0.025,0.975))*Sh  #bootstrap percentile t confidence interval
u0 + qt(c(0.025,0.975), n-1)*Sh    #student t confidence interval for comparison
