#01.23.11 22.00 chap7.r
#Copyright 2011 by Robert A LaBudde, all rights reserved
#Intro Bootstrap, Chap. 7
#created: 01.22.11 by r.a. labudde
#changes: 

set.seed(1) #for reproducibility
x<- cos(seq(-5,5,.2))
z<- sin(seq(-5,5,0.2))
ytrue<- 0.5 + 0.5*x + 0.2*x*x + 1*z + 0.1*x*z
e<- rnorm(51, 0, 0.5)
dat<- data.frame(x=x, x2=x*x, z=z, z2=z*z, xz=x*z, x3=x^3, z3=z^3, x2z=x*x*z, xz2=x*z*z, y=ytrue+e)
head(dat)
cor(dat)
n<- nrow(dat) #number data

#data fit
require('MASS')
fit1<- lm(y ~ x + z, data=dat)  #starting model
afit1<- stepAIC(fit1, scope=list(upper=~x+z+x2+z2+xz+x3+z3+x2z+xz2, lower=~1), trace=FALSE)
summary(afit1)

#bootstrap to find common models
nReal = 1000
p<- rep(0,9)  #proportions of each variable occurring
for (iReal in 1:nReal)  { #resample loop
  ind<- sample(1:n, n, replace=TRUE)  #bootstrap by indices (cases)
  bdat<- dat[ind,]  #pick out rows
  fit2<- lm(y ~ x + z, data=bdat)  #starting model
  afit2<- stepAIC(fit1, scope=list(upper=~x+z+x2+z2+xz+x3+z3+x2z+xz2, lower=~1), trace=FALSE)
  s<- names(coef(afit2))  #get fit variables
  m<- length(s)
  for (j in 2:m)  { #check terms for variables
    if (s[j]=='x') {
      p[1]<- p[1] + 1
    } else if (s[j]=='z') {
      p[2]<- p[2] + 1
    } else if (s[j]=='x2') {
      p[3]<- p[3] + 1
    } else if (s[j]=='z2') {
      p[4]<- p[4] + 1
    } else if (s[j]=='xz') {
      p[5]<- p[5] + 1
    } else if (s[j]=='x3') {
      p[6]<- p[6] + 1
    } else if (s[j]=='z3') {
      p[7]<- p[7] + 1
    } else if (s[j]=='x2z') {
      p[8]<- p[8] + 1
    } else if (s[j]=='xz2') {
      p[9]<- p[9] + 1
    } else {
      cat('Error! ', sprintf('%5d',m), sprintf('%5d',j), '\n')
      cat(s)
    }
  }
}
p<- p/nReal
p
  
#EXAMPLE 7.5.3
set.seed(1)   #for reproducibility
n<- c(45, 143, 70, 177, 116) #sample sizes
ncum<- cumsum(n)  #cumulative sum of sizes
x<- c(18, 58, 20, 51, 18)   #failures
N<- sum(n)    #total number data
X<- sum(x)    #total number failures

yA<- c(rep(1,18), rep(0,45-18))   #A data
yB<- c(rep(1,58), rep(0,143-58))  #B data
yC<- c(rep(1,20), rep(0,70-20))   #C data
yD<- c(rep(1,51), rep(0,177-51))  #D data
yE<- c(rep(1,26), rep(0,116-26))  #E data

p<- rep(0,4)
nReal<- 10000
for (iReal in 1:nReal) {  #resampling
  yyA<- sample(yA, n[1], replace=TRUE)  #A resample
  yyB<- sample(yB, n[2], replace=TRUE) #B resample
  yyC<- sample(yC, n[3], replace=TRUE)  #C resample
  yyD<- sample(yD, n[4], replace=TRUE) #D resample
  yyE<- sample(yE, n[5], replace=TRUE) #E resample
  pA<- sum(yyA)/n[1]    #A proportion
  pB<- sum(yyB)/n[2]   #B proportion
  pC<- sum(yyC)/n[3]    #C proportion
  pD<- sum(yyD)/n[4]   #D proportion
  pE<- sum(yyE)/n[5]   #E proportion
  if (pA<=pE) p[1]<- p[1] + 1  #count number of times A <= E
  if (pB<=pE) p[2]<- p[2] + 1  #count number of times B <= E
  if (pC<=pE) p[3]<- p[3] + 1  #count number of times C <= E
  if (pD<=pE) p[4]<- p[4] + 1  #count number of times D <= E
}
4*p/nReal   #Bonferroni-adjusted bootstrap P-values

y<- c(rep(1,X), rep(0,N-X))  #all data under null hypothesis
pnull<- rep(0,4)
nReal<- 1000
for (iReal in 1:nReal) {  #resampling
  yy<- sample(y, N, replace=TRUE) #sample for null
  pA<- sum(yy[1:ncum[1]])/n[1]           #A proportion
  pB<- sum(yy[(ncum[1]+1):ncum[2]])/n[2]   #B proportion
  pC<- sum(yy[(ncum[2]+1):ncum[3]])/n[3]    #C proportion
  pD<- sum(yy[(ncum[3]+1):ncum[4]])/n[4]   #D proportion
  pE<- sum(yy[(ncum[4]+1):ncum[5]])/n[5]   #E proportion
  if (pA>pE) pnull[1]<- pnull[1] + 1  #count number of times A > E
  if (pB>pE) pnull[2]<- pnull[2] + 1  #count number of times B > E
  if (pC>pE) pnull[3]<- pnull[3] + 1  #count number of times C > E
  if (pD>pE) pnull[4]<- pnull[4] + 1  #count number of times D > E
}
pnull/nReal

#EXAMPLE 7.7: HEART LESIONS
set.seed(1) #for reproducibility
x<- c(5.0,6.0,6.5,7.0,5.5,6.0,6.5,8.0,8.0,7.0,7.0,6.0,8.0,9.0,6.0,6.0,6.0, 
  6.0,7.0,7.0,6.0,7.0,6.0,6.0,7.0,6.0,7.0,7.0,7.0,7.0)  #lesion depths
length(x)
xm<- mean(x)
xs<- sd(x)
c(xm, xs, (10-xm)/(3*xs), (xm-4)/(3*xs))  #mean, std dev, CpkU, CpkL
qqnorm(x) #normal Q-Q plot
qqline(x)

require('boot')
bootCpk<- function(x, i) {
  m<- mean(x[i])
  s<- sd(x[i])
  CpkU<- (10-m)/(3*s)
  CpkL<- (m-4)/(3*s)
  return(min(CpkU, CpkL))
}
bCpk<- boot(x, bootCpk, R=10000)
boot.ci(bCpk)

#Surimi data
set.seed(1) #for reproducibility
x<- c(41.28, 45.16, 34.75, 40.76, 43.61, 39.05, 41.20, 41.02, 41.33, 40.61, 40.49, 41.77, 42.07, 
      44.83, 29.12, 45.59, 41.95, 45.78, 42.89, 40.42, 49.31, 44.01, 34.87, 38.60, 39.63, 38.52, 38.52,
      43.95, 49.08, 50.52, 43.85, 40.64, 45.86, 41.25, 50.35, 45.18, 39.67, 43.89, 43.89, 42.16)
length(x)
xm<- mean(x)
xs<- sd(x)
USL<- 60
LSL<- 20
c(xm, xs, (USL-xm)/(3*xs), (xm-LSL)/(3*xs))  #mean, std dev, CpkU, CpkL
qqnorm(x) #normal Q-Q plot
qqline(x)

require('boot')
bootCpk<- function(x, i) {
  m<- mean(x[i])
  s<- sd(x[i])
  CpkU<- (USL-m)/(3*s)
  CpkL<- (m-LSL)/(3*s)
  return(min(CpkU, CpkL))
}
bCpk<- boot(x, bootCpk, R=10000)
boot.ci(bCpk)
